# 陈一潇
# 1400017639


from pythonds.basic.stack import Stack
from pythonds.basic.queue import Queue


class NumStack(Stack):
    """定义了弹出计算最后两个数并压入回来的方法的栈"""

    def calc(self, token):
        """pop out the last two items, calculate them with token and push back to the stack"""
        op2=self.pop()
        op1=self.pop()

        if token == "^":
            self.push(op1 ** op2)
        elif token == "*":
            self.push(op1 * op2)
        elif token == "/":
            self.push(op1 / op2)
        elif token == "+":
            self.push(op1 + op2)
        elif token == "-":
            self.push(op1 - op2)
        else:
            raise RuntimeError("Wrong token!")


def infix_evaluation(infix_str):
    """中缀表达式求值，支持+-*/^()，支持小数计算，小数输出，可以随便加入空格，都会被忽略，左括号前不加符号默认为乘法"""
    prec = {"^": 4, "*": 3, "/": 3, "+": 2, "-": 2, "(": 1, ")": 1}
    op_stack = Stack()  # 存储操作符的栈
    num_stack = NumStack()  # 存储数字的栈
    temp_num_list = []  # 临时列表，用于储存数字字符方便之后拼合为数字

    pure_str = "".join(infix_str.split())  # 去除所有空格
    for char in pure_str:

        if char in "0123456789.":
            temp_num_list.append(char)  # 将数字字符暂存，以便遇到操作符时入栈

        # if temp_num_list:
        #     num_stack.push(float("".join(temp_num_list)))   # 数字入栈
        # temp_num_list = []  # 清空临时列表

        elif char == "(":
            if temp_num_list:  # 左括号前有数字，默认乘法
                num_stack.push(float("".join(temp_num_list)))   # 数字入栈
                temp_num_list = []  # 清空临时列表
                while not op_stack.isEmpty() and prec[op_stack.peek()] >= prec["*"]:
                    num_stack.calc(op_stack.pop())  # 计算
                op_stack.push("*")
            op_stack.push(char)

        elif char == ")":
            try:  # 避免错误表达式报错
                num_stack.push(float("".join(temp_num_list)))   # 数字入栈
                temp_num_list = []
            except:
                raise RuntimeError("Wrong expression! Please input infix expression!")
            token = op_stack.pop()
            while token != "(":
                num_stack.calc(token)  # 计算
                token = op_stack.pop()

        elif char in prec:
            if temp_num_list:
                num_stack.push(float("".join(temp_num_list)))   # 数字入栈
            temp_num_list = []  # 清空临时列表
            while not op_stack.isEmpty() and prec[op_stack.peek()] >= prec[char]:
                num_stack.calc(op_stack.pop())  # 计算
            op_stack.push(char)

        else:
            raise RuntimeError("Wrong expression!, Can only handle numbers and operators.")

    if temp_num_list:
        num_stack.push(float("".join(temp_num_list)))   # 最后的数字入栈

    while not op_stack.isEmpty():
        num_stack.calc(op_stack.pop())  # 计算剩下的所有

    if num_stack.size() != 1:
        raise RuntimeError("Wrong expression!")

    return num_stack.pop()

if __name__ == '__main__':  # 提供了一个测试接口,输入正确字符串才可以计算，否则返回错误代码
    while True:
        expression = input("input expression -> ")
        try:
            print(infix_evaluation(expression))
        except Exception as e:
            print(e)
